#This function calculates Harrell's C index and the Prioritized C index for diseases with 2 survival outcomes
#Two sets of competing risk scores are compared and the difference in Harrell's C index and the Prioritized C index is estimated
#Includes standard errors and confidence intervals

#Input:
#w1 = outcome-specific weight for 1st (highest priority) outcome
#w2 = outcome-specific weight for 2nd (highest priority) outcome
#d1 = event indicator that 1st outcome occurred at t1
#t1 = time for 1st outcome (min of time of 1st outcome and censoring time)
#d2 = event indicator that 2nd outcome occurred at t2
#t2 = time for 2nd outcome (min of time of 2nd outcome and censoring time)
#z11 = original risk score for 1st outcome
#z12 = original risk score for 2nd outcome
#tau = time to estimate C index to

#Output:
#Harrell's C index for 1st outcome using original risk score, SE, LCL, and UCL
#Number of subject pairs used in Harrell's C index for 1st outcome
#Harrell's C index for 2nd outcome using original risk score, SE, LCL, and UCL
#Number of subject pairs used in Harrell's C index for 2nd outcome
#Prioritized C index using original risk scores, SE, LCL, and UCL
#Number of subject pairs used in Prioritized C index using both outcomes
MSC_Cs <- function(w1,w2,d1,t1,d2,t2,z11,z12,tau1){
  
  #### calculate variance (based on methods by Kang et al., 2015)
  n <- length(d1)
  txy_ij1 <- matrix(0,n,n)  #for wa C - Z1
  txy_ij2 <- matrix(0,n,n)  #for wa C - Z2
  txx_ij <- matrix(0,n,n)   #for wa C - usable
  txy_ij11 <- matrix(0,n,n) #for C event 1 - Z1
  txy_ij12 <- matrix(0,n,n) #for C event 2 - Z1
  txx_ij1 <- matrix(0,n,n)  #for C event 1 - usable
  txx_ij2 <- matrix(0,n,n)  #for C event 2 - usable
  usable1 <- 0
  usable2 <- 0
  usable <- 0
  d1c <- 1-d1
  kmm1 <- survfit(Surv(t1,d1c)~ 1)  #kaplan-meier curve for censoring time distribution
  surv <- c(1,kmm1$surv)
  time <- c(0,kmm1$time)
  
  for (i in 1:n){
    print(i)
    for (j in i:n){
      sign11 <- I(z11[j] >= z11[i]) - I(z11[i] >= z11[j])   #predictor 1, event 1
      sign12 <- I(z12[j] >= z12[i]) - I(z12[i] >= z12[j])   #predictor 1, event 2
      csign1 <- d1[j]*I(t1[j] <= t1[i])*I(t1[j]<=tau1) - d1[i]*I(t1[i] <= t1[j])*I(t1[i]<=tau1)  #event 1, no IPW
      csign1_1 <- d1[j]*I(t1[j] <= t1[i])*I(t1[j]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t1[j],tau1)])^-1) - 
        d1[i]*I(t1[i] <= t1[j])*I(t1[i]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t1[i],tau1)])^-1)  #event 1, IPW for denominator
      csign1_2 <- d1[j]*I(t1[j] <= t1[i])*I(t1[j]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t1[j],tau1)])^-2) - 
        d1[i]*I(t1[i] <= t1[j])*I(t1[i]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t1[i],tau1)])^-2)  #event 1, IPW for numerator
      csign2 <- (d2[j]*I(t2[j] <= t2[i])*I(t2[j]<=tau1) - d2[i]*I(t2[i] <= t2[j])*I(t2[i]<=tau1))  #event 2, no IPW
      csign2_1 <- (d2[j]*I(t2[j] <= t2[i])*I(t2[j]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t2[j],tau1)])^-1) - 
                     d2[i]*I(t2[i] <= t2[j])*I(t2[i]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t2[i],tau1)])^-1))  #event 2, IPW for denominator
      csign2_2 <- (d2[j]*I(t2[j] <= t2[i])*I(t2[j]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t2[j],tau1)])^-2) - 
                     d2[i]*I(t2[i] <= t2[j])*I(t2[i]<=tau1)*(min(surv[time<=min(summary(kmm1)$time[length(summary(kmm1)$time)-1],t2[i],tau1)])^-2))  #event 2, IPW for numerator
      
      txy_ij1[i,j] <- w1*csign1_2*sign11 + w2*csign2_2*sign12*(1-csign1_1**2)  #numerator for C*, predictor 1
      txy_ij1[j,i] <- w1*csign1_2*sign11 + w2*csign2_2*sign12*(1-csign1_1**2)  #numerator for C*, predictor 1   
      txx_ij[i,j] <- w1*csign1_1**2 + w2*(csign2_1**2)*(1-csign1_1**2)   #denominator for C*
      txx_ij[j,i] <- w1*csign1_1**2 + w2*(csign2_1**2)*(1-csign1_1**2)   #denominator for C*    
      
      txy_ij11[i,j] <- csign1_2*sign11  #numerator for C, predictor 1, event 1
      txy_ij11[j,i] <- csign1_2*sign11  #numerator for C, predictor 1, event 1      
      txy_ij12[i,j] <- csign2_2*sign12  #numerator for C, predictor 1, event 2
      txy_ij12[j,i] <- csign2_2*sign12  #numerator for C, predictor 1, event 2  
      txx_ij1[i,j] <- csign1_1**2  #denominator for C, event 1
      txx_ij1[j,i] <- csign1_1**2  #denominator for C, event 1      
      txx_ij2[i,j] <- csign2_1**2  #denominator for C, event 2
      txx_ij2[j,i] <- csign2_1**2  #denominator for C, event 2    
      usable1 <- usable1 + csign1**2
      usable2 <- usable2 + csign2**2
      usable <- usable + max(csign1**2,csign2**2)
    }
  }    
  txy1 <- sum(txy_ij1)/(n*(n-1))
  txx <- sum(txx_ij)/(n*(n-1))   ### same as 2*theta
  kang_c1 <- .5*(txy1/txx + 1)
  
  var_txx <- (4*sum(rowSums(txx_ij)**2) - 2*sum(txx_ij**2) - 2*(2*n-3)/(n*(n-1))*sum(txx_ij)**2)/(n*(n-1)*(n-2)*(n-3))
  var_txy1 <- (4*sum(rowSums(txy_ij1)**2) - 2*sum(txy_ij1**2) - 2*(2*n-3)/(n*(n-1))*sum(txy_ij1)**2)/(n*(n-1)*(n-2)*(n-3))

  cov_txx_txy1 <- (4*sum(rowSums(txx_ij)*rowSums(txy_ij1)) - 2*sum(txx_ij*txy_ij1) - 2*(2*n-3)/(n*(n-1))*sum(txx_ij)*sum(txy_ij1))/(n*(n-1)*(n-2)*(n-3))

  var_kang_c1 <- .25*(var_txy1/(txx**2) - 2*txy1/(txx**3)*cov_txx_txy1 + (txy1/txx**2)**2*var_txx)

  txy11 <- sum(txy_ij11)/(n*(n-1))  #numerator for C, predictor 1, event 1
  txy12 <- sum(txy_ij12)/(n*(n-1))  #numerator for C, predictor 1, event 2

  txx1 <- sum(txx_ij1)/(n*(n-1))   ### same as 2*theta
  txx2 <- sum(txx_ij2)/(n*(n-1))   ### same as 2*theta
  kang_c11 <- .5*(txy11/txx1 + 1)  #C for predictor 1, event 1
  kang_c12 <- .5*(txy12/txx2 + 1)  #C for predictor 1, event 2

  var_txx1 <- (4*sum(rowSums(txx_ij1)**2) - 2*sum(txx_ij1**2) - 2*(2*n-3)/(n*(n-1))*sum(txx_ij1)**2)/(n*(n-1)*(n-2)*(n-3))
  var_txx2 <- (4*sum(rowSums(txx_ij2)**2) - 2*sum(txx_ij2**2) - 2*(2*n-3)/(n*(n-1))*sum(txx_ij2)**2)/(n*(n-1)*(n-2)*(n-3))
  var_txy11 <- (4*sum(rowSums(txy_ij11)**2) - 2*sum(txy_ij11**2) - 2*(2*n-3)/(n*(n-1))*sum(txy_ij11)**2)/(n*(n-1)*(n-2)*(n-3))
  var_txy12 <- (4*sum(rowSums(txy_ij12)**2) - 2*sum(txy_ij12**2) - 2*(2*n-3)/(n*(n-1))*sum(txy_ij12)**2)/(n*(n-1)*(n-2)*(n-3))

  cov_txx1_txy11 <- (4*sum(rowSums(txx_ij1)*rowSums(txy_ij11)) - 2*sum(txx_ij1*txy_ij11) - 2*(2*n-3)/(n*(n-1))*sum(txx_ij1)*sum(txy_ij11))/(n*(n-1)*(n-2)*(n-3))
  cov_txx2_txy12 <- (4*sum(rowSums(txx_ij2)*rowSums(txy_ij12)) - 2*sum(txx_ij2*txy_ij12) - 2*(2*n-3)/(n*(n-1))*sum(txx_ij2)*sum(txy_ij12))/(n*(n-1)*(n-2)*(n-3))
  
  var_kang_c11 <- .25*(var_txy11/(txx1**2)-2*txy11/(txx1**3)*cov_txx1_txy11+(txy11/txx1**2)**2*var_txx1)
  var_kang_c12 <- .25*(var_txy12/(txx2**2)-2*txy12/(txx2**3)*cov_txx2_txy12+(txy12/txx2**2)**2*var_txx2)

  results <- data.frame(kang_c11,sqrt(var_kang_c11),kang_c11-1.96*sqrt(var_kang_c11),kang_c11+1.96*sqrt(var_kang_c11),
                        usable1/2,
                        kang_c12,sqrt(var_kang_c12),kang_c12-1.96*sqrt(var_kang_c12),kang_c12+1.96*sqrt(var_kang_c12),
                        usable2/2,
                        kang_c1,sqrt(var_kang_c1),kang_c1-1.96*sqrt(var_kang_c1),kang_c1+1.96*sqrt(var_kang_c1),
                        usable/2)
  
  colnames(results) <- c("C Event 1/RS1","SE C Event 1/RS1","LCL C Event 1/RS1","UCL C Event 1/RS1",
                         "Usable Pairs",
                         "C Event 2/RS1","SE C Event 2/RS1","LCL C Event 2/RS1","UCL C Event 2/RS1",
                         "Usable Pairs",
                         "C*/RS1","SE C*/RS1","LCL C*/RS1","UCL C*/RS1",
                         "Usable Pairs")
  return(results)
}